@@ -57,10 +57,47 @@ class UFunc(Benchmark):
     def time_ufunc_types(self, ufuncname):
         [self.f(*arg) for arg in self.args]
 
+class UFuncSmall(Benchmark):
+    """  Benchmark for a selection of ufuncs on a small arrays and scalars 
+
+    Since the arrays and scalars are small, we are benchmarking the overhead 
+    of the numpy ufunc functionality
+    """
+    params = ['abs', 'sqrt', 'cos']
+    param_names = ['ufunc']
+    timeout = 10
+
+    def setup(self, ufuncname):
+        np.seterr(all='ignore')
+        try:
+            self.f = getattr(np, ufuncname)
+        except AttributeError:
+            raise NotImplementedError()
+        self.array_5 = np.array([1., 2., 10., 3., 4.])
+        self.array_int_3 = np.array([1, 2, 3])
+        self.float64 = np.float64(1.1)
+        self.python_float = 1.1
+        
+    def time_ufunc_small_array(self, ufuncname):
+        self.f(self.array_5)
+
+    def time_ufunc_small_array_inplace(self, ufuncname):
+        self.f(self.array_5, out = self.array_5)
+
+    def time_ufunc_small_int_array(self, ufuncname):
+        self.f(self.array_int_3)
+
+    def time_ufunc_numpy_scalar(self, ufuncname):
+        self.f(self.float64)
+
+    def time_ufunc_python_float(self, ufuncname):
+        self.f(self.python_float)
+        
 
 class Custom(Benchmark):
     def setup(self):
         self.b = np.ones(20000, dtype=bool)
+        self.b_small = np.ones(3, dtype=bool)
 
     def time_nonzero(self):
         np.nonzero(self.b)
@@ -74,6 +111,9 @@ class Custom(Benchmark):
     def time_or_bool(self):
         (self.b | self.b)
 
+    def time_and_bool_small(self):
+        (self.b_small & self.b_small)
+
 
 class CustomInplace(Benchmark):
     def setup(self):
@@ -130,8 +170,25 @@ class CustomScalar(Benchmark):
     def time_divide_scalar2_inplace(self, dtype):
         np.divide(self.d, 1, out=self.d)
 
+
+class CustomComparison(Benchmark):
+    params = (np.int8,  np.int16,  np.int32,  np.int64, np.uint8, np.uint16,
+              np.uint32, np.uint64, np.float32, np.float64, np.bool_)
+    param_names = ['dtype']
+
+    def setup(self, dtype):
+        self.x = np.ones(50000, dtype=dtype)
+        self.y = np.ones(50000, dtype=dtype)
+        self.s = np.ones(1, dtype=dtype)
+
+    def time_less_than_binary(self, dtype):
+        (self.x < self.y)
+
+    def time_less_than_scalar1(self, dtype):
+        (self.s < self.x)
+
     def time_less_than_scalar2(self, dtype):
-        (self.d < 1)
+        (self.x < self.s)
 
 
 class CustomScalarFloorDivideInt(Benchmark):
@@ -150,6 +207,19 @@ class CustomScalarFloorDivideInt(Benchmark):
     def time_floor_divide_int(self, dtype, divisor):
         self.x // divisor
 
+class CustomArrayFloorDivideInt(Benchmark):
+    params = (np.sctypes['int'] + np.sctypes['uint'], [100, 10000, 1000000])
+    param_names = ['dtype', 'size']
+
+    def setup(self, dtype, size):
+        iinfo = np.iinfo(dtype)
+        self.x = np.random.randint(
+                    iinfo.min, iinfo.max, size=size, dtype=dtype)
+        self.y = np.random.randint(2, 32, size=size, dtype=dtype)
+
+    def time_floor_divide_int(self, dtype, size):
+        self.x // self.y
+
 
 class Scalar(Benchmark):
     def setup(self):
@@ -224,3 +294,23 @@ class ArgParsingReduce(Benchmark):
 
     def time_add_reduce_arg_parsing(self, arg_pack):
         np.add.reduce(*arg_pack.args, **arg_pack.kwargs)
+
+class BinaryBench(Benchmark):
+    def setup(self):
+        N = 1000000
+        self.a32 = np.random.rand(N).astype(np.float32)
+        self.b32 = np.random.rand(N).astype(np.float32)
+        self.a64 = np.random.rand(N).astype(np.float64)
+        self.b64 = np.random.rand(N).astype(np.float64)
+    
+    def time_pow_32(self):
+        np.power(self.a32, self.b32)
+
+    def time_pow_64(self):
+        np.power(self.a64, self.b64)
+
+    def time_atan2_32(self):
+        np.arctan2(self.a32, self.b32)
+
+    def time_atan2_64(self):
+        np.arctan2(self.a64, self.b64)
