@@ -1,6 +1,7 @@
 """
 Scan the directory of nep files and extract their metadata.  The
-metadata is passed to Jinja for filling out `index.rst.tmpl`.
+metadata is passed to Jinja for filling out the toctrees for various NEP
+categories.
 """
 
 import os
@@ -22,6 +23,7 @@ def nep_metadata():
 
     meta_re = r':([a-zA-Z\-]*): (.*)'
 
+    has_provisional = False
     neps = {}
     print('Loading metadata for:')
     for source in sources:
@@ -49,7 +51,7 @@ def nep_metadata():
         if not tags['Title'].startswith(f'NEP {nr} — '):
             raise RuntimeError(
                 f'Title for NEP {nr} does not start with "NEP {nr} — " '
-                '(note that — here is a special, enlongated dash). Got: '
+                '(note that — here is a special, elongated dash). Got: '
                 f'    {tags["Title"]!r}')
 
         if tags['Status'] in ('Accepted', 'Rejected', 'Withdrawn'):
@@ -58,6 +60,8 @@ def nep_metadata():
                     f'NEP {nr} is Accepted/Rejected/Withdrawn but '
                     'has no Resolution tag'
                 )
+        if tags['Status'] == 'Provisional':
+            has_provisional = True
 
         neps[nr] = tags
 
@@ -95,16 +99,18 @@ def nep_metadata():
                     f'been set to Superseded'
                 )
 
-    return {'neps': neps}
-
-
-infile = 'index.rst.tmpl'
-outfile = 'index.rst'
+    return {'neps': neps, 'has_provisional': has_provisional}
 
 meta = nep_metadata()
 
-print(f'Compiling {infile} -> {outfile}')
-index = render(infile, meta)
+for nepcat in (
+    "provisional", "accepted", "deferred", "finished", "meta",
+    "open", "rejected",
+):
+    infile = f"{nepcat}.rst.tmpl"
+    outfile =f"{nepcat}.rst"
 
-with open(outfile, 'w') as f:
-    f.write(index)
+    print(f'Compiling {infile} -> {outfile}')
+    genf = render(infile, meta)
+    with open(outfile, 'w') as f:
+        f.write(genf)
