@@ -39,8 +39,6 @@ if __name__ == "__main__":
     config = configuration(top_path='')
     config = config.todict()
 
-    config['download_url'] = "http://cens.ioc.ee/projects/f2py2e/2.x"\
-                             "/F2PY-2-latest.tar.gz"
     config['classifiers'] = [
         'Development Status :: 5 - Production/Stable',
         'Intended Audience :: Developers',
@@ -68,6 +66,6 @@ command line tool (f2py) for generating Python C/API modules for
 wrapping Fortran 77/90/95 subroutines, accessing common blocks from
 Python, and calling Python functions from Fortran (call-backs).
 Interfacing subroutines/data from Fortran 90/95 modules is supported.""",
-          url="http://cens.ioc.ee/projects/f2py2e/",
+          url="https://numpy.org/doc/stable/f2py/",
           keywords=['Fortran', 'f2py'],
           **config)
