@@ -192,10 +192,16 @@ def main(argv):
         site_dir, site_dir_noarch = build_project(args)
         sys.path.insert(0, site_dir)
         sys.path.insert(0, site_dir_noarch)
-        os.environ['PYTHONPATH'] = site_dir + os.pathsep + site_dir_noarch
+        os.environ['PYTHONPATH'] = \
+            os.pathsep.join((
+                site_dir, 
+                site_dir_noarch, 
+                os.environ.get('PYTHONPATH', '')
+            ))
     else:
-        _temp = __import__(PROJECT_MODULE)
-        site_dir = os.path.sep.join(_temp.__file__.split(os.path.sep)[:-2])
+        if not args.bench_compare:
+            _temp = __import__(PROJECT_MODULE)
+            site_dir = os.path.sep.join(_temp.__file__.split(os.path.sep)[:-2])
 
     extra_argv = args.args[:]
     if not args.bench:
@@ -281,6 +287,8 @@ def main(argv):
     if args.refguide_check:
         cmd = [os.path.join(ROOT_DIR, 'tools', 'refguide_check.py'),
                '--doctests']
+        if args.verbose:
+            cmd += ['-' + 'v'*args.verbose]
         if args.submodule:
             cmd += [args.submodule]
         os.execv(sys.executable, [sys.executable] + cmd)
@@ -474,23 +482,18 @@ def build_project(args):
             '--single-version-externally-managed',
             '--record=' + dst_dir + 'tmp_install_log.txt']
 
-    py_v_s = sysconfig.get_config_var('py_version_short')
-    platlibdir = getattr(sys, 'platlibdir', '')  # Python3.9+
+    config_vars = dict(sysconfig.get_config_vars())
+    config_vars["platbase"] = dst_dir
+    config_vars["base"] = dst_dir
+
     site_dir_template = os.path.normpath(sysconfig.get_path(
         'platlib', expand=False
     ))
-    site_dir = site_dir_template.format(platbase=dst_dir,
-                                        py_version_short=py_v_s,
-                                        platlibdir=platlibdir,
-                                        base=dst_dir,
-                                        )
+    site_dir = site_dir_template.format(**config_vars)
     noarch_template = os.path.normpath(sysconfig.get_path(
         'purelib', expand=False
     ))
-    site_dir_noarch = noarch_template.format(base=dst_dir,
-                                             py_version_short=py_v_s,
-                                             platlibdir=platlibdir,
-                                             )
+    site_dir_noarch = noarch_template.format(**config_vars)
 
     # easy_install won't install to a path that Python by default cannot see
     # and isn't on the PYTHONPATH.  Plus, it has to exist.
@@ -498,7 +501,8 @@ def build_project(args):
         os.makedirs(site_dir)
     if not os.path.exists(site_dir_noarch):
         os.makedirs(site_dir_noarch)
-    env['PYTHONPATH'] = site_dir + os.pathsep + site_dir_noarch
+    env['PYTHONPATH'] = \
+        os.pathsep.join((site_dir, site_dir_noarch, env.get('PYTHONPATH', '')))
 
     log_filename = os.path.join(ROOT_DIR, 'build.log')
 
@@ -624,7 +628,7 @@ def asv_substitute_config(in_config, out_config, **custom_vars):
             hash_line = wfd.readline().split('hash:')
             if len(hash_line) > 1 and int(hash_line[1]) == vars_hash:
                 return True
-    except IOError:
+    except OSError:
         pass
 
     custom_vars = {f'{{{k}}}':v for k, v in custom_vars.items()}
